"""Webhook 通知：推送统计摘要到飞书/钉钉/Slack"""

from __future__ import annotations

import json
import urllib.request
import urllib.error
from datetime import datetime, timedelta, timezone
from pathlib import Path

from .analyzer import SessionStats, analyze_session, merge_stats
from .parser import find_sessions, parse_jsonl


def _collect_today_stats() -> SessionStats | None:
    """收集今天的统计数据"""
    today_start = datetime.now(tz=timezone.utc).replace(
        hour=0, minute=0, second=0, microsecond=0
    )
    all_files = find_sessions()
    today_stats = []

    for f in all_files:
        try:
            session = parse_jsonl(f)
            stats = analyze_session(session)
            if stats.end_time and stats.end_time >= today_start:
                today_stats.append(stats)
        except Exception:
            continue

    if not today_stats:
        return None
    return merge_stats(today_stats) if len(today_stats) > 1 else today_stats[0]


def _estimate_cost(stats: SessionStats) -> float:
    MODEL_PRICING = {
        "claude-opus-4": {"input": 15, "output": 75, "cache_read": 1.5, "cache_create": 18.75},
        "claude-sonnet-4": {"input": 3, "output": 15, "cache_read": 0.3, "cache_create": 3.75},
        "claude-haiku-4": {"input": 0.8, "output": 4, "cache_read": 0.08, "cache_create": 1},
    }
    cost = 0.0
    for model, usage in stats.token_by_model.items():
        p = MODEL_PRICING.get("claude-opus-4")  # default
        lower = model.lower()
        for key in MODEL_PRICING:
            if key.split("-")[1] in lower:
                p = MODEL_PRICING[key]
                break
        if p:
            cost += usage.input_tokens / 1e6 * p["input"]
            cost += usage.output_tokens / 1e6 * p["output"]
            cost += usage.cache_read_input_tokens / 1e6 * p["cache_read"]
            cost += usage.cache_creation_input_tokens / 1e6 * p["cache_create"]
    return cost


def _fmt_tokens(n: int) -> str:
    if n >= 1_000_000_000:
        return f"{n / 1e9:.1f}B"
    if n >= 1_000_000:
        return f"{n / 1e6:.1f}M"
    if n >= 1_000:
        return f"{n / 1e3:.1f}K"
    return str(n)


def _fmt_duration(seconds: float) -> str:
    h = int(seconds) // 3600
    m = (int(seconds) % 3600) // 60
    if h > 0:
        return f"{h}h {m}m"
    return f"{m}m"


def _build_message(stats: SessionStats) -> dict:
    """构建通知消息内容"""
    cost = _estimate_cost(stats)
    today = datetime.now().strftime("%Y-%m-%d")
    active = stats.active_duration.total_seconds()
    ai_pct = (
        round(stats.ai_duration.total_seconds() / active * 100)
        if active > 0 else 0
    )

    # 效率评分
    total_tokens = stats.token_usage.total
    total_code = stats.total_added + stats.total_removed
    code_per_1k = round(total_code / max(total_tokens / 1000, 1), 2) if total_tokens > 0 else 0
    avg_tpm = total_tokens // max(stats.user_message_count, 1)
    code_score = min(40, int(code_per_1k / 0.5 * 40))
    precision_score = max(0, min(30, int((1 - min(avg_tpm, 200_000) / 200_000) * 30)))
    util_score = min(30, int(ai_pct / 70 * 30))
    total_score = code_score + precision_score + util_score
    grade = "S" if total_score >= 90 else "A" if total_score >= 75 else "B" if total_score >= 60 else "C" if total_score >= 40 else "D"

    return {
        "date": today,
        "sessions": len([s for s in [stats]]),  # merged = 1
        "instructions": stats.user_message_count,
        "active_time": _fmt_duration(active),
        "ai_ratio": f"{ai_pct}%",
        "tokens": _fmt_tokens(total_tokens),
        "cost": f"${cost:.2f}",
        "code_added": stats.total_added + stats.git_total_added,
        "code_removed": stats.total_removed + stats.git_total_removed,
        "grade": grade,
        "score": total_score,
        "git_commits": stats.git_commit_count,
    }


def send_feishu(webhook_url: str, stats: SessionStats) -> bool:
    """发送飞书机器人通知"""
    msg = _build_message(stats)
    payload = {
        "msg_type": "interactive",
        "card": {
            "header": {
                "title": {"tag": "plain_text", "content": f"Claude Code 日报 {msg['date']}"},
                "template": "blue",
            },
            "elements": [
                {
                    "tag": "div",
                    "fields": [
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**会话数**\n{msg['instructions']} 条指令"}},
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**活跃时长**\n{msg['active_time']} (AI {msg['ai_ratio']})"}},
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**Token**\n{msg['tokens']}"}},
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**费用**\n{msg['cost']}"}},
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**代码**\n+{msg['code_added']} / -{msg['code_removed']}"}},
                        {"is_short": True, "text": {"tag": "lark_md", "content": f"**效率**\n{msg['grade']} ({msg['score']}/100)"}},
                    ],
                },
            ],
        },
    }
    return _post_json(webhook_url, payload)


def send_dingtalk(webhook_url: str, stats: SessionStats) -> bool:
    """发送钉钉机器人通知"""
    msg = _build_message(stats)
    text = (
        f"### Claude Code 日报 {msg['date']}\n\n"
        f"| 指标 | 数值 |\n"
        f"|------|------|\n"
        f"| 指令数 | {msg['instructions']} |\n"
        f"| 活跃时长 | {msg['active_time']} (AI {msg['ai_ratio']}) |\n"
        f"| Token | {msg['tokens']} |\n"
        f"| 费用 | {msg['cost']} |\n"
        f"| 代码 | +{msg['code_added']} / -{msg['code_removed']} |\n"
        f"| 效率 | {msg['grade']} ({msg['score']}/100) |\n"
        f"| Git | {msg['git_commits']} commits |"
    )
    payload = {"msgtype": "markdown", "markdown": {"title": "Claude Code 日报", "text": text}}
    return _post_json(webhook_url, payload)


def send_slack(webhook_url: str, stats: SessionStats) -> bool:
    """发送 Slack 通知"""
    msg = _build_message(stats)
    text = (
        f"*Claude Code Daily Report {msg['date']}*\n"
        f"• Instructions: {msg['instructions']} | Active: {msg['active_time']} (AI {msg['ai_ratio']})\n"
        f"• Tokens: {msg['tokens']} | Cost: {msg['cost']}\n"
        f"• Code: +{msg['code_added']} / -{msg['code_removed']} | {msg['git_commits']} commits\n"
        f"• Efficiency: {msg['grade']} ({msg['score']}/100)"
    )
    payload = {"text": text}
    return _post_json(webhook_url, payload)


def _post_json(url: str, payload: dict) -> bool:
    """POST JSON 到 webhook URL"""
    data = json.dumps(payload, ensure_ascii=False).encode()
    req = urllib.request.Request(
        url, data=data,
        headers={"Content-Type": "application/json"},
        method="POST",
    )
    try:
        with urllib.request.urlopen(req, timeout=10) as resp:
            return resp.status == 200
    except (urllib.error.URLError, OSError) as e:
        print(f"Webhook 发送失败: {e}")
        return False


def send_notification(webhook_url: str, platform: str = "auto") -> bool:
    """发送今日统计通知

    Args:
        webhook_url: Webhook URL
        platform: feishu/dingtalk/slack/auto (自动检测)
    """
    stats = _collect_today_stats()
    if not stats:
        print("今天暂无会话数据")
        return False

    # 自动检测平台
    if platform == "auto":
        if "feishu.cn" in webhook_url or "larksuite.com" in webhook_url:
            platform = "feishu"
        elif "dingtalk.com" in webhook_url or "oapi.dingtalk" in webhook_url:
            platform = "dingtalk"
        elif "hooks.slack.com" in webhook_url:
            platform = "slack"
        else:
            print("无法自动检测平台，请指定 --platform feishu/dingtalk/slack")
            return False

    senders = {
        "feishu": send_feishu,
        "dingtalk": send_dingtalk,
        "slack": send_slack,
    }
    sender = senders.get(platform)
    if not sender:
        print(f"不支持的平台: {platform}")
        return False

    if sender(webhook_url, stats):
        print(f"已发送到 {platform}")
        return True
    return False



"""cc-stats-web: start local web dashboard and open browser"""

import threading
import webbrowser

from .server import start_server


def main():
    server, port = start_server()
    url = f"http://127.0.0.1:{port}/"
    print(f"CC Stats Web Dashboard: {url}")
    print("Press Ctrl+C to stop.")

    # Open browser after short delay
    threading.Timer(0.5, lambda: webbrowser.open(url)).start()

    try:
        server.serve_forever()
    except KeyboardInterrupt:
        print("\nStopped.")
        server.shutdown()


if __name__ == "__main__":
    main()



"""cc-stats-app 入口：启动 SwiftUI 菜单栏 App

优先使用预编译二进制（从 GitHub Release 下载），没有则 fallback 到本地 swiftc 编译。
"""

import glob
import json
import os
import platform
import subprocess
import sys
import urllib.request
import urllib.error

_swift_dir = os.path.join(os.path.dirname(__file__), "swift")
_swift_bin = os.path.join(_swift_dir, "CCStats")
_version_file = os.path.join(_swift_dir, ".binary_version")

GITHUB_REPO = "androidZzT/cc-statistics"


def _get_current_version() -> str:
    """从 pyproject.toml 或 SettingsView 读取当前版本"""
    settings = os.path.join(_swift_dir, "Views", "SettingsView.swift")
    if os.path.exists(settings):
        with open(settings) as f:
            for line in f:
                if "appVersion" in line and "=" in line:
                    return line.split('"')[1]
    return "unknown"


def _get_binary_version() -> str:
    """读取已下载二进制的版本"""
    if os.path.exists(_version_file):
        with open(_version_file) as f:
            return f.read().strip()
    return ""


def _save_binary_version(version: str):
    with open(_version_file, "w") as f:
        f.write(version)


def _try_download_binary() -> bool:
    """尝试从 GitHub Release 下载预编译二进制"""
    # 如果本地源码比二进制新，跳过预编译，走本地编译
    if _need_recompile():
        return False

    current_version = _get_current_version()
    binary_version = _get_binary_version()

    # 已有匹配版本的二进制且文件存在
    if (
        binary_version == current_version
        and os.path.exists(_swift_bin)
        and os.access(_swift_bin, os.X_OK)
    ):
        return True

    arch = platform.machine()  # arm64 or x86_64
    if arch not in ("arm64", "x86_64"):
        return False

    asset_name = f"CCStats-{arch}"
    tag = f"v{current_version}"

    try:
        # 查询 release assets
        url = f"https://api.github.com/repos/{GITHUB_REPO}/releases/tags/{tag}"
        req = urllib.request.Request(url, headers={"Accept": "application/vnd.github+json"})
        with urllib.request.urlopen(req, timeout=10) as resp:
            release = json.loads(resp.read().decode())

        # 找到对应架构的 asset
        asset_url = None
        for asset in release.get("assets", []):
            if asset["name"] == asset_name:
                asset_url = asset["browser_download_url"]
                break

        if not asset_url:
            return False

        print(f"Downloading prebuilt binary ({arch})...")
        urllib.request.urlretrieve(asset_url, _swift_bin)
        os.chmod(_swift_bin, 0o755)
        _save_binary_version(current_version)
        print("Done.")
        return True

    except (urllib.error.URLError, json.JSONDecodeError, OSError, KeyError):
        return False


def _need_recompile() -> bool:
    """检查是否需要重新编译"""
    if not os.path.exists(_swift_bin):
        return True
    bin_mtime = os.path.getmtime(_swift_bin)
    for swift_file in glob.glob(os.path.join(_swift_dir, "**", "*.swift"), recursive=True):
        if os.path.getmtime(swift_file) > bin_mtime:
            return True
    return False


def _compile_swift():
    """编译 SwiftUI 菜单栏 App（仅首次或源码更新时）"""
    if not _need_recompile():
        return

    print("Compiling CCStats app...")

    # 收集所有 Swift 文件
    swift_files = glob.glob(os.path.join(_swift_dir, "**", "*.swift"), recursive=True)

    result = subprocess.run(
        [
            "swiftc",
            *swift_files,
            "-o", _swift_bin,
            "-framework", "Cocoa",
            "-framework", "SwiftUI",
            "-framework", "Carbon",
            "-lsqlite3",
            "-O",
        ],
        capture_output=True,
        text=True,
    )
    if result.returncode != 0:
        print(f"Swift compilation failed:\n{result.stderr}", file=sys.stderr)
        sys.exit(1)
    print("Done.")


def main():
    # 优先下载预编译二进制
    if not _try_download_binary():
        # fallback: 本地编译
        _compile_swift()

    # 自动后台运行：fork 进程后父进程退出，不占用终端
    if os.fork() != 0:
        # 父进程：打印提示后退出
        print("CCStats is running in the background.")
        sys.exit(0)

    # 子进程：脱离终端
    os.setsid()

    # 关闭标准输入输出
    devnull = os.open(os.devnull, os.O_RDWR)
    os.dup2(devnull, 0)
    os.dup2(devnull, 1)
    os.dup2(devnull, 2)
    os.close(devnull)

    # 启动 Swift app
    os.execv(_swift_bin, [_swift_bin])


if __name__ == "__main__":
    main()



"""格式化统计结果输出（带 ANSI 色彩）"""

from __future__ import annotations

import os
from datetime import timedelta

from .analyzer import TOOL_DESCRIPTIONS, SessionStats


# ── ANSI 色彩 ──────────────────────────────────────────────
def _supports_color() -> bool:
    """检测终端是否支持色彩"""
    if os.environ.get("NO_COLOR"):
        return False
    if os.environ.get("FORCE_COLOR"):
        return True
    return hasattr(os, "isatty") and os.isatty(1)


_COLOR = _supports_color()


def _c(code: str, text: str) -> str:
    """给文本加 ANSI 色彩"""
    if not _COLOR:
        return text
    return f"\033[{code}m{text}\033[0m"


# 常用颜色快捷方式
def _bold(t: str) -> str: return _c("1", t)
def _dim(t: str) -> str: return _c("2", t)
def _cyan(t: str) -> str: return _c("36", t)
def _green(t: str) -> str: return _c("32", t)
def _red(t: str) -> str: return _c("31", t)
def _yellow(t: str) -> str: return _c("33", t)
def _blue(t: str) -> str: return _c("34", t)
def _magenta(t: str) -> str: return _c("35", t)
def _white_bold(t: str) -> str: return _c("1;37", t)
def _cyan_bold(t: str) -> str: return _c("1;36", t)
def _green_bold(t: str) -> str: return _c("1;32", t)


# ── 格式化辅助 ─────────────────────────────────────────────

def _fmt_duration(td: timedelta) -> str:
    """将 timedelta 格式化为可读字符串"""
    total_seconds = int(td.total_seconds())
    if total_seconds < 0:
        return "0s"
    hours, remainder = divmod(total_seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    parts = []
    if hours:
        parts.append(f"{hours}h")
    if minutes:
        parts.append(f"{minutes}m")
    if seconds or not parts:
        parts.append(f"{seconds}s")
    return " ".join(parts)


def _fmt_tokens(n: int) -> str:
    """格式化 token 数量"""
    if n >= 1_000_000:
        return f"{n / 1_000_000:.1f}M"
    if n >= 1_000:
        return f"{n / 1_000:.1f}K"
    return str(n)


def _bar(value: int, max_value: int, width: int = 20) -> str:
    """生成带颜色的柱状图"""
    if max_value == 0:
        return ""
    filled = int(value / max_value * width)
    bar_filled = "█" * filled
    bar_empty = "░" * (width - filled)
    return _cyan(bar_filled) + _dim(bar_empty)


def _net_str(net: int) -> str:
    """格式化净增数（带颜色）"""
    if net > 0:
        return _green(f"+{net}")
    elif net < 0:
        return _red(str(net))
    return _dim("0")


# ── 主格式化 ───────────────────────────────────────────────

def format_stats(stats: SessionStats, session_count: int = 1) -> str:
    """将统计结果格式化为终端输出"""
    lines: list[str] = []
    sep = _dim("─" * 60)

    # ── Header ──
    lines.append("")
    lines.append(_cyan("  ╔══════════════════════════════════════════════════════════╗"))
    lines.append(_cyan("  ║") + _white_bold("        Claude Code 会话统计报告") + "                   " + _cyan("║"))
    lines.append(_cyan("  ╚══════════════════════════════════════════════════════════╝"))
    lines.append("")

    if stats.project_path and stats.project_path != "all":
        lines.append(f"  {_dim('项目:')} {_bold(stats.project_path)}")
    if session_count > 1:
        lines.append(f"  {_dim('会话数:')} {_bold(str(session_count))}")
    if stats.start_time:
        start_local = stats.start_time.astimezone()
        end_local = stats.end_time.astimezone() if stats.end_time else None
        end_str = end_local.strftime('%Y-%m-%d %H:%M') if end_local else '?'
        lines.append(f"  {_dim('时间范围:')} {start_local.strftime('%Y-%m-%d %H:%M')} ~ {end_str}")
    lines.append("")

    # ── ① 用户指令数 ──
    lines.append(f"  {_cyan_bold('①')} {_bold('用户指令数')}")
    lines.append(sep)
    lines.append(f"  对话轮次: {_yellow(str(stats.user_message_count))}")
    lines.append("")

    # ── ② AI 工具调用 ──
    lines.append(f"  {_cyan_bold('②')} {_bold('AI 工具调用')}")
    lines.append(sep)
    lines.append(f"  总调用次数: {_yellow(str(stats.tool_call_total))}")
    lines.append("")
    if stats.tool_call_counts:
        sorted_tools = sorted(
            stats.tool_call_counts.items(), key=lambda x: x[1], reverse=True
        )
        max_count = sorted_tools[0][1] if sorted_tools else 1
        max_name_len = max(len(name) for name, _ in sorted_tools)

        for name, count in sorted_tools:
            desc = TOOL_DESCRIPTIONS.get(name, "")
            bar = _bar(count, max_count, 15)
            desc_part = f"  {_dim(desc)}" if desc else ""
            lines.append(
                f"  {_bold(f'{name:<{max_name_len}}')}  {bar} {_yellow(f'{count:>5}')}{desc_part}"
            )
    lines.append("")

    # ── ③ 开发时长 ──
    lines.append(f"  {_cyan_bold('③')} {_bold('开发时长')}")
    lines.append(sep)
    lines.append(f"  活跃时长:     {_green_bold(_fmt_duration(stats.active_duration))}")
    lines.append(f"    {_blue('AI 处理:')}    {_blue(_fmt_duration(stats.ai_duration))}")
    lines.append(f"    {_magenta('用户活跃:')}  {_magenta(_fmt_duration(stats.user_duration))}")
    if stats.active_duration.total_seconds() > 0:
        ai_ratio = stats.ai_duration.total_seconds() / stats.active_duration.total_seconds() * 100
        lines.append(f"  AI 占比:      {_blue(f'{ai_ratio:.0f}%')}")
    if stats.turn_count:
        avg_ai = stats.ai_duration / stats.turn_count
        lines.append(f"  平均轮次耗时: {_fmt_duration(avg_ai)}/轮 {_dim(f'({stats.turn_count} 轮)')}")
    lines.append("")

    # ── ④ 代码变更 ──
    lines.append(f"  {_cyan_bold('④')} {_bold('代码变更')}")
    lines.append(sep)

    if stats.git_available:
        git_net = stats.git_total_added - stats.git_total_removed
        ai_pct = (
            f" ({stats.git_ai_commit_count}/{stats.git_commit_count} AI)"
            if stats.git_ai_commit_count > 0
            else ""
        )
        lines.append(f"  {_yellow('[Git 已提交]')}  {stats.git_commit_count} 个 commit{_cyan(ai_pct)}")
        lines.append(f"  总新增: {_green(f'+{stats.git_total_added}')}  总删除: {_red(f'-{stats.git_total_removed}')}  净增: {_net_str(git_net)}")
        if stats.git_ai_commit_count > 0:
            ai_code_pct = round(
                (stats.git_ai_added + stats.git_ai_removed)
                / max(stats.git_total_added + stats.git_total_removed, 1)
                * 100
            )
            lines.append(
                f"  AI 代码: {_cyan(f'+{stats.git_ai_added}')} {_cyan(f'-{stats.git_ai_removed}')}  "
                f"占比: {_cyan(f'{ai_code_pct}%')}"
            )
        lines.append("")

        if stats.git_lines_by_lang:
            sorted_langs = sorted(
                stats.git_lines_by_lang.items(),
                key=lambda x: x[1]["added"] + x[1]["removed"],
                reverse=True,
            )
            max_lang_len = max(len(lang) for lang, _ in sorted_langs)
            for lang, counts in sorted_langs:
                added = counts["added"]
                removed = counts["removed"]
                net_l = added - removed
                lines.append(
                    f"  {_dim(f'{lang:<{max_lang_len}}')}  {_green(f'+{added:<6}')} {_red(f'-{removed:<6}')} net {_net_str(net_l)}"
                )
        lines.append("")

    ai_net = stats.total_added - stats.total_removed
    lines.append(f"  {_blue('[AI 工具变更]')}  {_dim('来自 Edit/Write 调用')}")
    lines.append(f"  总新增: {_green(f'+{stats.total_added}')}  总删除: {_red(f'-{stats.total_removed}')}  净增: {_net_str(ai_net)}")
    lines.append("")

    if stats.lines_by_lang:
        sorted_langs = sorted(
            stats.lines_by_lang.items(),
            key=lambda x: x[1]["added"] + x[1]["removed"],
            reverse=True,
        )
        max_lang_len = max(len(lang) for lang, _ in sorted_langs)
        for lang, counts in sorted_langs:
            added = counts["added"]
            removed = counts["removed"]
            net_l = added - removed
            lines.append(
                f"  {_dim(f'{lang:<{max_lang_len}}')}  {_green(f'+{added:<6}')} {_red(f'-{removed:<6}')} net {_net_str(net_l)}"
            )
    lines.append("")

    # ── ⑤ Token 消耗 ──
    lines.append(f"  {_cyan_bold('⑤')} {_bold('Token 消耗')}")
    lines.append(sep)
    tu = stats.token_usage
    lines.append(f"  Input tokens:          {_fmt_tokens(tu.input_tokens):>10}")
    lines.append(f"  Output tokens:         {_yellow(_fmt_tokens(tu.output_tokens)):>22}")
    lines.append(f"  Cache read tokens:     {_dim(_fmt_tokens(tu.cache_read_input_tokens)):>22}")
    lines.append(f"  Cache creation tokens: {_dim(_fmt_tokens(tu.cache_creation_input_tokens)):>22}")
    lines.append(f"  {_dim('─' * 40)}")
    lines.append(f"  合计:                  {_white_bold(_fmt_tokens(tu.total)):>22}")
    lines.append("")

    if stats.token_by_model:
        lines.append(f"  {_dim('按模型拆分:')}")
        for model, usage in sorted(stats.token_by_model.items()):
            if usage.total == 0:
                continue
            lines.append(
                f"    {_cyan(model)}: "
                f"input={_fmt_tokens(usage.input_tokens)} "
                f"output={_yellow(_fmt_tokens(usage.output_tokens))} "
                f"cache_read={_dim(_fmt_tokens(usage.cache_read_input_tokens))} "
                f"total={_bold(_fmt_tokens(usage.total))}"
            )
    lines.append("")

    # ⑥ 效率评分
    total_tokens = stats.token_usage.total
    total_code = stats.total_added + stats.total_removed
    if total_tokens > 0 and stats.user_message_count > 0:
        avg_tokens_per_msg = total_tokens // stats.user_message_count
        code_per_1k = round(total_code / max(total_tokens / 1000, 1), 2)
        active_secs = stats.active_duration.total_seconds()
        ai_secs = stats.ai_duration.total_seconds()
        ai_ratio = round(ai_secs / max(active_secs, 1) * 100)

        code_score = min(40, int(code_per_1k / 0.5 * 40))
        precision_score = max(0, min(30, int((1 - min(avg_tokens_per_msg, 200_000) / 200_000) * 30)))
        util_score = min(30, int(ai_ratio / 70 * 30))
        total_score = code_score + precision_score + util_score
        grade = "S" if total_score >= 90 else "A" if total_score >= 75 else "B" if total_score >= 60 else "C" if total_score >= 40 else "D"

        grade_color = _green if grade in ("S", "A") else _yellow if grade == "B" else _red
        lines.append(f"  {_bold('⑥ 效率评分')}")
        lines.append("─" * 60)
        lines.append(f"  评分: {grade_color(f'{grade} ({total_score}/100)')}")
        lines.append(f"  代码产出: {_fmt_tokens(total_code)} 行 / {_fmt_tokens(total_tokens)} Token = {_cyan(f'{code_per_1k} 行/K')}")
        lines.append(f"  指令精准: {_fmt_tokens(avg_tokens_per_msg)} Token/条")
        lines.append(f"  AI 利用率: {ai_ratio}%")
        lines.append("")

    return "\n".join(lines)



"""CLI 入口"""

from __future__ import annotations

import argparse
import sys
from datetime import datetime, timedelta, timezone
from pathlib import Path

from .analyzer import analyze_session, merge_stats
from .formatter import format_stats
from .parser import find_sessions, find_sessions_by_keyword, parse_jsonl


def _parse_time_arg(value: str) -> datetime:
    """解析时间参数，支持多种格式：

    绝对时间:
      2026-03-13
      2026-03-13T10:00
      2026-03-13T10:00:00

    相对时间 (相对于当前时刻):
      1h    → 1 小时前
      3d    → 3 天前
      2w    → 2 周前
    """
    value = value.strip()

    # 相对时间
    if value and value[-1] in ("h", "d", "w") and value[:-1].isdigit():
        n = int(value[:-1])
        unit = value[-1]
        delta = {"h": timedelta(hours=n), "d": timedelta(days=n), "w": timedelta(weeks=n)}[unit]
        return datetime.now(tz=timezone.utc) - delta

    # 绝对时间（视为本地时间）
    for fmt in ("%Y-%m-%d", "%Y-%m-%dT%H:%M", "%Y-%m-%dT%H:%M:%S"):
        try:
            dt = datetime.strptime(value, fmt)
            return dt.astimezone(timezone.utc)
        except ValueError:
            continue

    raise argparse.ArgumentTypeError(
        f"无法解析时间: {value}（支持 2026-03-13, 2026-03-13T10:00, 3d, 2w, 1h）"
    )


def _resolve_project_name(proj_dir: Path, jsonl_files: list[Path]) -> str:
    """从 JSONL 文件中的 cwd 字段还原项目真实路径"""
    import json
    for jf in jsonl_files:
        with open(jf, encoding="utf-8") as fh:
            for ln in fh:
                try:
                    obj = json.loads(ln)
                    if obj.get("cwd"):
                        return obj["cwd"]
                except (json.JSONDecodeError, UnicodeDecodeError):
                    continue
    # fallback: 目录名本身
    return proj_dir.name


def _compare_projects(args) -> None:
    """对比所有项目的关键指标"""
    from .formatter import _fmt_duration, _fmt_tokens

    claude_projects = Path.home() / ".claude" / "projects"
    if not claude_projects.exists():
        print("未找到 Claude Code 项目数据")
        return

    projects: list[dict] = []

    for proj in sorted(claude_projects.iterdir()):
        if not proj.is_dir():
            continue
        jsonl_files = list(proj.glob("*.jsonl"))
        if not jsonl_files:
            continue

        name = _resolve_project_name(proj, jsonl_files)
        # 简化路径显示
        short_name = Path(name).name if "/" in name else name

        all_stats = []
        for f in jsonl_files:
            try:
                session = parse_jsonl(f)
                stats = analyze_session(session)

                # 时间过滤
                if args.since and stats.end_time and stats.end_time < args.since:
                    continue
                if args.until and stats.start_time and stats.start_time > args.until:
                    continue

                all_stats.append(stats)
            except Exception:
                continue

        if not all_stats:
            continue

        merged = merge_stats(all_stats) if len(all_stats) > 1 else all_stats[0]

        from .reporter import _estimate_cost
        cost = _estimate_cost(merged)

        projects.append({
            "name": short_name,
            "sessions": len(all_stats),
            "instructions": merged.user_message_count,
            "duration": merged.active_duration,
            "tokens": merged.token_usage.total,
            "cost": cost,
            "added": merged.total_added + merged.git_total_added,
            "removed": merged.total_removed + merged.git_total_removed,
            "grade": merged.efficiencyGrade if hasattr(merged, 'efficiencyGrade') else "",
        })

    if not projects:
        print("没有项目数据")
        return

    # 按 token 总量降序排列
    projects.sort(key=lambda p: p["tokens"], reverse=True)

    # 计算列宽
    max_name = max(len(p["name"]) for p in projects)
    max_name = max(max_name, 4)  # 最小宽度

    # 表头
    print()
    print(f"  {'项目':<{max_name}}  {'会话':>4}  {'指令':>5}  {'活跃时长':>10}  {'Token':>8}  {'费用':>8}  {'代码':>10}")
    print("─" * (max_name + 60))

    total_sessions = 0
    total_instructions = 0
    total_tokens = 0
    total_cost = 0.0

    for p in projects:
        dur_str = _fmt_duration(p["duration"])
        tok_str = _fmt_tokens(p["tokens"])
        cost_str = f"${p['cost']:.0f}" if p["cost"] >= 1 else f"${p['cost']:.2f}"
        code_str = f"+{p['added']}/-{p['removed']}"

        print(f"  {p['name']:<{max_name}}  {p['sessions']:>4}  {p['instructions']:>5}  {dur_str:>10}  {tok_str:>8}  {cost_str:>8}  {code_str:>10}")

        total_sessions += p["sessions"]
        total_instructions += p["instructions"]
        total_tokens += p["tokens"]
        total_cost += p["cost"]

    print("─" * (max_name + 60))
    print(f"  {'合计':<{max_name}}  {total_sessions:>4}  {total_instructions:>5}  {'':>10}  {_fmt_tokens(total_tokens):>8}  ${total_cost:>7.0f}")
    print()


def _list_projects() -> None:
    """列出所有已知项目"""
    claude_projects = Path.home() / ".claude" / "projects"
    if not claude_projects.exists():
        print("未找到 Claude Code 项目数据")
        return

    print("\n可用项目:")
    print("─" * 60)
    for proj in sorted(claude_projects.iterdir()):
        if not proj.is_dir():
            continue
        jsonl_files = list(proj.glob("*.jsonl"))
        if not jsonl_files:
            continue
        display_name = _resolve_project_name(proj, jsonl_files)
        print(f"  {display_name}  ({len(jsonl_files)} 个会话)")
    print()


def main(argv: list[str] | None = None) -> None:
    parser = argparse.ArgumentParser(
        prog="cc-stats",
        description="Claude Code 会话统计工具 — 分析 AI Coding 工程指标",
    )
    parser.add_argument(
        "path",
        nargs="?",
        help="JSONL 文件路径，或项目目录路径。不指定则分析当前目录的所有会话。",
    )
    parser.add_argument(
        "--all",
        action="store_true",
        help="分析所有项目的所有会话",
    )
    parser.add_argument(
        "--list",
        action="store_true",
        dest="list_projects",
        help="列出所有已知项目",
    )
    parser.add_argument(
        "--last",
        type=int,
        metavar="N",
        help="只分析最近 N 个会话",
    )
    parser.add_argument(
        "--since",
        type=_parse_time_arg,
        metavar="TIME",
        help="只包含此时间之后的会话（如 2026-03-13, 3d, 2w, 1h）",
    )
    parser.add_argument(
        "--until",
        type=_parse_time_arg,
        metavar="TIME",
        help="只包含此时间之前的会话（如 2026-03-14, 1d）",
    )

    parser.add_argument(
        "--report",
        choices=["week", "month"],
        metavar="PERIOD",
        help="生成周报(week)或月报(month)，输出 Markdown 格式",
    )
    parser.add_argument(
        "--compare",
        action="store_true",
        help="对比所有项目的关键指标",
    )
    parser.add_argument(
        "--notify",
        metavar="WEBHOOK_URL",
        help="发送今日统计到 Webhook（自动检测飞书/钉钉/Slack）",
    )
    parser.add_argument(
        "--platform",
        choices=["feishu", "dingtalk", "slack"],
        help="指定 Webhook 平台（配合 --notify 使用）",
    )

    args = parser.parse_args(argv)

    if args.report:
        from .reporter import generate_report
        print(generate_report(args.report))
        return

    if args.notify:
        from .webhook import send_notification
        send_notification(args.notify, args.platform or "auto")
        return

    if args.compare:
        _compare_projects(args)
        return

    if args.list_projects:
        _list_projects()
        return

    # 确定要分析的 JSONL 文件
    jsonl_files: list[Path] = []

    if args.path:
        p = Path(args.path)
        if p.is_file() and p.suffix == ".jsonl":
            jsonl_files = [p]
        elif p.is_dir():
            jsonl_files = find_sessions(p)
        if not jsonl_files:
            # 作为关键词模糊搜索（同时搜索目录名和 JSONL 中的 cwd）
            jsonl_files = find_sessions_by_keyword(args.path)
        if not jsonl_files:
            print(f"找不到: {args.path}", file=sys.stderr)
            sys.exit(1)
    elif args.all:
        jsonl_files = find_sessions()
    else:
        # 默认：当前目录
        jsonl_files = find_sessions(Path.cwd())

    if not jsonl_files:
        print("未找到 JSONL 会话文件。使用 --list 查看可用项目。", file=sys.stderr)
        sys.exit(1)

    # 按修改时间排序
    jsonl_files.sort(key=lambda f: f.stat().st_mtime)

    if args.last:
        jsonl_files = jsonl_files[-args.last:]

    # 解析 & 分析（按时间范围过滤）
    all_stats = []
    for f in jsonl_files:
        session = parse_jsonl(f)
        stats = analyze_session(session)

        # --since: 跳过结束时间在 since 之前的会话
        if args.since and stats.end_time and stats.end_time < args.since:
            continue
        # --until: 跳过开始时间在 until 之后的会话
        if args.until and stats.start_time and stats.start_time > args.until:
            continue

        all_stats.append(stats)

    if not all_stats:
        print("指定时间范围内无会话。", file=sys.stderr)
        sys.exit(1)

    if len(all_stats) == 1:
        result = all_stats[0]
    else:
        result = merge_stats(all_stats)

    print(format_stats(result, session_count=len(all_stats)))


if __name__ == "__main__":
    main()



"""解析 Claude Code JSONL 会话文件"""

from __future__ import annotations

import json
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any


@dataclass
class ToolCall:
    name: str
    input: dict[str, Any]
    timestamp: str


@dataclass
class Message:
    role: str  # "user" | "assistant"
    timestamp: str
    content: Any
    model: str | None = None
    usage: dict[str, Any] = field(default_factory=dict)
    tool_calls: list[ToolCall] = field(default_factory=list)
    is_tool_result: bool = False
    is_meta: bool = False
    session_id: str = ""


@dataclass
class Session:
    session_id: str
    project_path: str
    file_path: Path
    messages: list[Message] = field(default_factory=list)


def parse_jsonl(path: Path) -> Session:
    """解析单个 JSONL 文件为 Session 对象"""
    messages: list[Message] = []
    session_id = path.stem
    project_path = ""

    with open(path, encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            if not line:
                continue
            try:
                obj = json.loads(line)
            except json.JSONDecodeError:
                continue

            msg_type = obj.get("type")
            if msg_type not in ("user", "assistant"):
                continue

            if not project_path:
                project_path = obj.get("cwd", "")

            raw_msg = obj.get("message", {})
            timestamp = obj.get("timestamp", "")
            content = raw_msg.get("content", "")

            # 判断是否为 tool_result（工具返回）
            is_tool_result = False
            if isinstance(content, list):
                for block in content:
                    if isinstance(block, dict) and block.get("type") == "tool_result":
                        is_tool_result = True
                        break

            # 提取 tool_use 调用
            tool_calls: list[ToolCall] = []
            if msg_type == "assistant" and isinstance(content, list):
                for block in content:
                    if isinstance(block, dict) and block.get("type") == "tool_use":
                        tool_calls.append(ToolCall(
                            name=block.get("name", ""),
                            input=block.get("input", {}),
                            timestamp=timestamp,
                        ))

            messages.append(Message(
                role=msg_type,
                timestamp=timestamp,
                content=content,
                model=raw_msg.get("model"),
                usage=raw_msg.get("usage", {}),
                tool_calls=tool_calls,
                is_tool_result=is_tool_result,
                is_meta=obj.get("isMeta", False),
                session_id=obj.get("sessionId", session_id),
            ))

    return Session(
        session_id=session_id,
        project_path=project_path,
        file_path=path,
        messages=messages,
    )


def _path_to_dirname(path: Path) -> str:
    """将绝对路径转为 Claude Code 的项目目录名格式

    例如 /Users/foo/bar → -Users-foo-bar
    """
    return str(path.resolve()).replace("/", "-")


def find_sessions(project_dir: Path | None = None) -> list[Path]:
    """查找 ~/.claude/projects/ 下所有 JSONL 会话文件

    如果指定 project_dir，只返回匹配的项目。
    """
    claude_projects = Path.home() / ".claude" / "projects"
    if not claude_projects.exists():
        return []

    results: list[Path] = []
    target_dirname = _path_to_dirname(project_dir) if project_dir else None

    for proj in sorted(claude_projects.iterdir()):
        if not proj.is_dir():
            continue
        if target_dirname:
            if proj.name != target_dirname:
                continue
        for jsonl in sorted(proj.glob("*.jsonl")):
            # 跳过子代理会话
            if jsonl.name.startswith("agent-"):
                continue
            results.append(jsonl)

    return results


def find_sessions_by_keyword(keyword: str) -> list[Path]:
    """按关键词模糊匹配项目，在目录名和 JSONL 中的 cwd 中搜索"""
    import json

    claude_projects = Path.home() / ".claude" / "projects"
    if not claude_projects.exists():
        return []

    results: list[Path] = []
    keyword_lower = keyword.lower()

    for proj in sorted(claude_projects.iterdir()):
        if not proj.is_dir():
            continue
        jsonl_files = sorted(proj.glob("*.jsonl"))
        if not jsonl_files:
            continue

        # 先在目录名中搜索
        if keyword_lower in proj.name.lower():
            results.extend(jsonl_files)
            continue

        # 再在 JSONL 的 cwd 中搜索
        for jf in jsonl_files[:1]:
            try:
                with open(jf, encoding="utf-8") as fh:
                    for ln in fh:
                        try:
                            obj = json.loads(ln)
                            cwd = obj.get("cwd", "")
                            if cwd and keyword_lower in cwd.lower():
                                results.extend(jsonl_files)
                                break
                        except (json.JSONDecodeError, UnicodeDecodeError):
                            continue
                    else:
                        continue
                    break  # matched, stop checking more files
            except OSError:
                continue

    return results



"""HTTP server: static files + JSON API"""

from __future__ import annotations

import json
import os
import socket
from collections import defaultdict
from datetime import datetime, timedelta, timezone
from http.server import HTTPServer, SimpleHTTPRequestHandler
from urllib.parse import parse_qs, urlparse

from cc_stats.analyzer import SessionStats, TokenUsage, analyze_session, merge_stats
from cc_stats.parser import find_sessions, parse_jsonl

_web_dir = os.path.join(os.path.dirname(__file__), "web")

# Model pricing ($/M tokens)
_PRICING = {
    "opus": {"input": 15, "output": 75, "cache_read": 1.5, "cache_create": 18.75},
    "sonnet": {"input": 3, "output": 15, "cache_read": 0.3, "cache_create": 3.75},
    "haiku": {"input": 0.8, "output": 4, "cache_read": 0.08, "cache_create": 1.0},
    "gpt-4o": {"input": 2.5, "output": 10, "cache_read": 1.25, "cache_create": 2.5},
    "o1": {"input": 15, "output": 60, "cache_read": 7.5, "cache_create": 15},
    "o3": {"input": 10, "output": 40, "cache_read": 2.5, "cache_create": 10},
}


def _match_pricing(model: str) -> dict:
    lower = model.lower()
    for key in ["opus", "haiku", "sonnet", "gpt-4o", "o1", "o3"]:
        if key in lower:
            return _PRICING[key]
    return _PRICING["sonnet"]


def _estimate_cost(tu: TokenUsage, model: str = "") -> float:
    p = _match_pricing(model)
    cost = 0.0
    cost += tu.input_tokens / 1e6 * p["input"]
    cost += tu.output_tokens / 1e6 * p["output"]
    cost += tu.cache_read_input_tokens / 1e6 * p["cache_read"]
    cost += tu.cache_creation_input_tokens / 1e6 * p["cache_create"]
    return cost


def _resolve_project_name(proj_dir, jsonl_files):
    for jf in jsonl_files:
        try:
            with open(jf, encoding="utf-8") as fh:
                for ln in fh:
                    try:
                        obj = json.loads(ln)
                        if obj.get("cwd"):
                            return obj["cwd"]
                    except (json.JSONDecodeError, UnicodeDecodeError):
                        continue
        except OSError:
            continue
    return proj_dir.name


def _stats_to_dict(stats: SessionStats, session_count: int = 1) -> dict:
    def _td_seconds(td):
        return td.total_seconds()

    def _fmt_duration(td):
        total = int(td.total_seconds())
        if total < 0:
            return "0s"
        h, rem = divmod(total, 3600)
        m, s = divmod(rem, 60)
        parts = []
        if h:
            parts.append(f"{h}h")
        if m:
            parts.append(f"{m}m")
        if s or not parts:
            parts.append(f"{s}s")
        return " ".join(parts)

    def _token_dict(tu):
        return {
            "input_tokens": tu.input_tokens,
            "output_tokens": tu.output_tokens,
            "cache_read": tu.cache_read_input_tokens,
            "cache_creation": tu.cache_creation_input_tokens,
            "total": tu.total,
        }

    sorted_tools = sorted(stats.tool_call_counts.items(), key=lambda x: x[1], reverse=True)
    sorted_langs = sorted(stats.lines_by_lang.items(), key=lambda x: x[1]["added"], reverse=True)

    # Cost per model
    total_cost = 0.0
    model_tokens = []
    for model, usage in sorted(stats.token_by_model.items(), key=lambda x: x[1].total, reverse=True):
        cost = _estimate_cost(usage, model)
        total_cost += cost
        model_tokens.append({
            "model": model,
            **_token_dict(usage),
            "cost": round(cost, 4),
        })

    return {
        "session_count": session_count,
        "user_message_count": stats.user_message_count,
        "tool_call_total": stats.tool_call_total,
        "tool_calls": [{"name": n, "count": c} for n, c in sorted_tools],
        "total_duration": _td_seconds(stats.total_duration),
        "total_duration_fmt": _fmt_duration(stats.total_duration),
        "ai_duration": _td_seconds(stats.ai_duration),
        "ai_duration_fmt": _fmt_duration(stats.ai_duration),
        "user_duration": _td_seconds(stats.user_duration),
        "user_duration_fmt": _fmt_duration(stats.user_duration),
        "active_duration": _td_seconds(stats.active_duration),
        "active_duration_fmt": _fmt_duration(stats.active_duration),
        "turn_count": stats.turn_count,
        "total_added": stats.total_added,
        "total_removed": stats.total_removed,
        "lines_by_lang": [{"lang": l, **c} for l, c in sorted_langs],
        "git_available": stats.git_available,
        "git_total_added": stats.git_total_added,
        "git_total_removed": stats.git_total_removed,
        "git_commit_count": stats.git_commit_count,
        "token_usage": _token_dict(stats.token_usage),
        "token_by_model": model_tokens,
        "estimated_cost": round(total_cost, 2),
    }


def _get_projects():
    from pathlib import Path
    claude_projects = Path.home() / ".claude" / "projects"
    if not claude_projects.exists():
        return []
    projects = []
    for proj in sorted(claude_projects.iterdir()):
        if not proj.is_dir():
            continue
        jsonl_files = [f for f in proj.glob("*.jsonl") if not f.name.startswith("agent-")]
        if not jsonl_files:
            continue
        display_name = _resolve_project_name(proj, jsonl_files)
        projects.append({
            "dir_name": proj.name,
            "display_name": display_name,
            "session_count": len(jsonl_files),
        })
    projects.sort(key=lambda x: x["session_count"], reverse=True)
    return projects


def _get_stats(project_dir_name=None, since_days=None):
    from pathlib import Path
    claude_projects = Path.home() / ".claude" / "projects"

    if project_dir_name:
        proj_dir = claude_projects / project_dir_name
        jsonl_files = sorted(f for f in proj_dir.glob("*.jsonl") if not f.name.startswith("agent-"))
    else:
        jsonl_files = [f for f in find_sessions() if not f.name.startswith("agent-")]

    if not jsonl_files:
        return {"error": "No sessions found"}

    jsonl_files.sort(key=lambda f: f.stat().st_mtime)

    since_dt = None
    if since_days:
        since_dt = datetime.now(tz=timezone.utc) - timedelta(days=since_days)

    all_stats = []
    for f in jsonl_files:
        try:
            session = parse_jsonl(f)
            stats = analyze_session(session)
            if since_dt and stats.end_time and stats.end_time < since_dt:
                continue
            all_stats.append(stats)
        except Exception:
            continue

    if not all_stats:
        return {"error": "No valid sessions"}

    result = all_stats[0] if len(all_stats) == 1 else merge_stats(all_stats)
    return _stats_to_dict(result, session_count=len(all_stats))


def _get_daily_stats(project_dir_name=None, days=14):
    from pathlib import Path
    claude_projects = Path.home() / ".claude" / "projects"

    if project_dir_name:
        proj_dir = claude_projects / project_dir_name
        jsonl_files = sorted(f for f in proj_dir.glob("*.jsonl") if not f.name.startswith("agent-"))
    else:
        jsonl_files = [f for f in find_sessions() if not f.name.startswith("agent-")]

    since_dt = datetime.now(tz=timezone.utc) - timedelta(days=days)
    daily: dict[str, list] = defaultdict(list)

    for f in jsonl_files:
        try:
            session = parse_jsonl(f)
            stats = analyze_session(session)
            if stats.end_time and stats.end_time < since_dt:
                continue
            if not stats.start_time:
                continue
            day_key = stats.start_time.astimezone().strftime("%Y-%m-%d")
            daily[day_key].append(stats)
        except Exception:
            continue

    result = []
    today = datetime.now().date()
    for i in range(days - 1, -1, -1):
        d = today - timedelta(days=i)
        day_key = d.strftime("%Y-%m-%d")
        day_stats = daily.get(day_key, [])
        if day_stats:
            merged = merge_stats(day_stats) if len(day_stats) > 1 else day_stats[0]
            cost = sum(_estimate_cost(u, m) for m, u in merged.token_by_model.items())
            result.append({
                "date": day_key,
                "sessions": len(day_stats),
                "messages": merged.user_message_count,
                "tool_calls": merged.tool_call_total,
                "active_minutes": round(merged.active_duration.total_seconds() / 60, 1),
                "lines_added": merged.total_added,
                "lines_removed": merged.total_removed,
                "tokens": merged.token_usage.total,
                "cost": round(cost, 2),
            })
        else:
            result.append({
                "date": day_key, "sessions": 0, "messages": 0, "tool_calls": 0,
                "active_minutes": 0, "lines_added": 0, "lines_removed": 0, "tokens": 0, "cost": 0,
            })
    return result


class ApiHandler(SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, directory=_web_dir, **kwargs)

    def do_GET(self):
        parsed = urlparse(self.path)
        path = parsed.path
        params = parse_qs(parsed.query)

        if path == "/api/projects":
            self._json(_get_projects())
        elif path == "/api/stats":
            project = params.get("project", [None])[0]
            days = params.get("days", [None])[0]
            self._json(_get_stats(
                project_dir_name=project or None,
                since_days=int(days) if days and days != "0" else None,
            ))
        elif path == "/api/daily_stats":
            project = params.get("project", [None])[0]
            days = params.get("days", ["14"])[0]
            self._json(_get_daily_stats(
                project_dir_name=project or None,
                days=int(days),
            ))
        else:
            super().do_GET()

    def _json(self, data):
        body = json.dumps(data, ensure_ascii=False).encode("utf-8")
        self.send_response(200)
        self.send_header("Content-Type", "application/json; charset=utf-8")
        self.send_header("Content-Length", str(len(body)))
        self.end_headers()
        self.wfile.write(body)

    def log_message(self, format, *args):
        pass


def find_free_port() -> int:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


def start_server() -> tuple[HTTPServer, int]:
    port = find_free_port()
    server = HTTPServer(("127.0.0.1", port), ApiHandler)
    return server, port



"""生成 Markdown 周报/月报"""

from __future__ import annotations

from collections import defaultdict
from datetime import datetime, timedelta, timezone
from pathlib import Path

from .analyzer import SessionStats, merge_stats, analyze_session
from .parser import find_sessions, parse_jsonl

# 模型定价 ($/M tokens)
_PRICING = {
    "opus": {"input": 15, "output": 75, "cache_read": 1.5, "cache_create": 18.75},
    "sonnet": {"input": 3, "output": 15, "cache_read": 0.3, "cache_create": 3.75},
    "haiku": {"input": 0.8, "output": 4, "cache_read": 0.08, "cache_create": 1.0},
}


def _match_pricing(model: str) -> dict:
    lower = model.lower()
    for key in ["opus", "haiku", "sonnet"]:
        if key in lower:
            return _PRICING[key]
    return _PRICING["sonnet"]


def _estimate_cost(stats: SessionStats) -> float:
    cost = 0.0
    for model, usage in stats.token_by_model.items():
        p = _match_pricing(model)
        cost += usage.input_tokens / 1e6 * p["input"]
        cost += usage.output_tokens / 1e6 * p["output"]
        cost += usage.cache_read_input_tokens / 1e6 * p["cache_read"]
        cost += usage.cache_creation_input_tokens / 1e6 * p["cache_create"]
    return cost


def _fmt_duration(td: timedelta) -> str:
    total = int(td.total_seconds())
    if total < 0:
        return "0s"
    h, rem = divmod(total, 3600)
    m, s = divmod(rem, 60)
    parts = []
    if h:
        parts.append(f"{h}h")
    if m:
        parts.append(f"{m}m")
    if s or not parts:
        parts.append(f"{s}s")
    return " ".join(parts)


def _fmt_tokens(n: int) -> str:
    if n >= 1e9:
        return f"{n / 1e9:.1f}B"
    if n >= 1e6:
        return f"{n / 1e6:.1f}M"
    if n >= 1e3:
        return f"{n / 1e3:.1f}K"
    return str(n)


def _fmt_cost(n: float) -> str:
    if n >= 100:
        return f"${n:.0f}"
    if n >= 1:
        return f"${n:.2f}"
    return f"${n:.3f}"


def generate_report(period: str = "week") -> str:
    """生成周报或月报 Markdown

    Args:
        period: "week" 或 "month"
    """
    now = datetime.now(tz=timezone.utc)
    if period == "month":
        since = now - timedelta(days=30)
        title = "月报"
        title_en = "Monthly Report"
        days = 30
    else:
        since = now - timedelta(days=7)
        title = "周报"
        title_en = "Weekly Report"
        days = 7

    start_str = since.astimezone().strftime("%Y-%m-%d")
    end_str = now.astimezone().strftime("%Y-%m-%d")

    # 收集所有会话
    jsonl_files = [f for f in find_sessions() if not f.name.startswith("agent-")]
    jsonl_files.sort(key=lambda f: f.stat().st_mtime)

    all_stats: list[SessionStats] = []
    daily: dict[str, list[SessionStats]] = defaultdict(list)

    for f in jsonl_files:
        try:
            session = parse_jsonl(f)
            stats = analyze_session(session)
            if stats.end_time and stats.end_time < since:
                continue
            all_stats.append(stats)
            if stats.start_time:
                day_key = stats.start_time.astimezone().strftime("%Y-%m-%d")
                daily[day_key].append(stats)
        except Exception:
            continue

    if not all_stats:
        return f"# Claude Code {title} ({start_str} ~ {end_str})\n\n> 该时段无会话数据。\n"

    merged = merge_stats(all_stats) if len(all_stats) > 1 else all_stats[0]
    cost = _estimate_cost(merged)

    # 按项目分组
    project_stats: dict[str, list[SessionStats]] = defaultdict(list)
    for s in all_stats:
        proj = s.project_path or "Unknown"
        proj_name = Path(proj).name if proj != "all" else proj
        project_stats[proj_name].append(s)

    # 每日统计
    daily_lines = []
    today = datetime.now().date()
    for i in range(days - 1, -1, -1):
        d = today - timedelta(days=i)
        day_key = d.strftime("%Y-%m-%d")
        day_stats_list = daily.get(day_key, [])
        if day_stats_list:
            ds = merge_stats(day_stats_list) if len(day_stats_list) > 1 else day_stats_list[0]
            dc = _estimate_cost(ds)
            daily_lines.append(
                f"| {day_key} | {len(day_stats_list)} | {ds.user_message_count} | "
                f"{_fmt_duration(ds.active_duration)} | {_fmt_tokens(ds.token_usage.total)} | {_fmt_cost(dc)} |"
            )

    # 工具调用 Top 5
    sorted_tools = sorted(merged.tool_call_counts.items(), key=lambda x: x[1], reverse=True)[:5]

    # 语言统计
    sorted_langs = sorted(merged.lines_by_lang.items(), key=lambda x: x[1]["added"], reverse=True)[:5]

    # 生成 Markdown
    lines = [
        f"# Claude Code {title}",
        f"",
        f"**{start_str} ~ {end_str}**",
        f"",
        f"## 概览",
        f"",
        f"| 指标 | 数值 |",
        f"|------|------|",
        f"| 会话数 | {len(all_stats)} |",
        f"| 指令数 | {merged.user_message_count} |",
        f"| 工具调用 | {merged.tool_call_total} |",
        f"| 活跃时长 | {_fmt_duration(merged.active_duration)} |",
        f"| AI 处理 | {_fmt_duration(merged.ai_duration)} |",
        f"| 用户活跃 | {_fmt_duration(merged.user_duration)} |",
        f"| Token 消耗 | {_fmt_tokens(merged.token_usage.total)} |",
        f"| 预估费用 | {_fmt_cost(cost)} |",
        f"| 代码新增 | +{merged.total_added} |",
        f"| 代码删除 | -{merged.total_removed} |",
    ]

    if merged.git_available:
        ai_info = ""
        if merged.git_ai_commit_count > 0:
            ai_pct = round(merged.git_ai_commit_count / max(merged.git_commit_count, 1) * 100)
            ai_info = f" ({merged.git_ai_commit_count} AI, {ai_pct}%)"
        lines.append(f"| Git Commits | {merged.git_commit_count}{ai_info} |")

    lines += [
        f"",
        f"## 每日明细",
        f"",
        f"| 日期 | 会话 | 指令 | 活跃时长 | Token | 费用 |",
        f"|------|------|------|----------|-------|------|",
    ]
    lines.extend(daily_lines)

    if sorted_tools:
        lines += [
            f"",
            f"## 工具调用 Top 5",
            f"",
            f"| 工具 | 次数 |",
            f"|------|------|",
        ]
        for name, count in sorted_tools:
            lines.append(f"| {name} | {count} |")

    if sorted_langs:
        lines += [
            f"",
            f"## 代码变更（按语言）",
            f"",
            f"| 语言 | 新增 | 删除 | 净增 |",
            f"|------|------|------|------|",
        ]
        for lang, counts in sorted_langs:
            net = counts["added"] - counts["removed"]
            sign = "+" if net >= 0 else ""
            lines.append(f"| {lang} | +{counts['added']} | -{counts['removed']} | {sign}{net} |")

    if merged.token_by_model:
        lines += [
            f"",
            f"## Token 消耗（按模型）",
            f"",
            f"| 模型 | Token | 费用 |",
            f"|------|-------|------|",
        ]
        for model, usage in sorted(merged.token_by_model.items(), key=lambda x: x[1].total, reverse=True):
            mc = 0.0
            p = _match_pricing(model)
            mc += usage.input_tokens / 1e6 * p["input"]
            mc += usage.output_tokens / 1e6 * p["output"]
            mc += usage.cache_read_input_tokens / 1e6 * p["cache_read"]
            mc += usage.cache_creation_input_tokens / 1e6 * p["cache_create"]
            lines.append(f"| {model} | {_fmt_tokens(usage.total)} | {_fmt_cost(mc)} |")

    if len(project_stats) > 1:
        lines += [
            f"",
            f"## 项目分布",
            f"",
            f"| 项目 | 会话 | 指令 | 费用 |",
            f"|------|------|------|------|",
        ]
        for proj_name, stats_list in sorted(project_stats.items(), key=lambda x: len(x[1]), reverse=True)[:10]:
            pm = merge_stats(stats_list) if len(stats_list) > 1 else stats_list[0]
            pc = _estimate_cost(pm)
            lines.append(f"| {proj_name} | {len(stats_list)} | {pm.user_message_count} | {_fmt_cost(pc)} |")

    # 对比上一周期
    prev_since = since - timedelta(days=days)
    prev_stats: list[SessionStats] = []
    for f in jsonl_files:
        try:
            session = parse_jsonl(f)
            stats_item = analyze_session(session)
            if stats_item.end_time and prev_since <= stats_item.end_time < since:
                prev_stats.append(stats_item)
        except Exception:
            continue

    if prev_stats:
        prev_merged = merge_stats(prev_stats) if len(prev_stats) > 1 else prev_stats[0]
        prev_cost = _estimate_cost(prev_merged)

        def _delta(curr, prev, fmt_fn=str):
            if prev == 0:
                return "—"
            pct = (curr - prev) / prev * 100
            sign = "+" if pct >= 0 else ""
            return f"{fmt_fn(curr)} ({sign}{pct:.0f}%)"

        prev_title = f"上{title[0]}对比" if "周" in title or "月" in title else "Previous Period"
        lines += [
            f"",
            f"## {prev_title}",
            f"",
            f"| 指标 | 本{title[0]} | 上{title[0]} | 变化 |",
            f"|------|------|------|------|",
            f"| 会话 | {len(all_stats)} | {len(prev_stats)} | {_delta(len(all_stats), len(prev_stats))} |",
            f"| 指令 | {merged.user_message_count} | {prev_merged.user_message_count} | {_delta(merged.user_message_count, prev_merged.user_message_count)} |",
            f"| Token | {_fmt_tokens(merged.token_usage.total)} | {_fmt_tokens(prev_merged.token_usage.total)} | {_delta(merged.token_usage.total, prev_merged.token_usage.total, _fmt_tokens)} |",
            f"| 费用 | {_fmt_cost(cost)} | {_fmt_cost(prev_cost)} | {_delta(cost, prev_cost, _fmt_cost)} |",
            f"| 代码新增 | +{merged.total_added} | +{prev_merged.total_added} | {_delta(merged.total_added, prev_merged.total_added)} |",
        ]

    # 成本预测
    active_days = len([d for d in daily_lines if d])  # 有数据的天数
    if active_days > 0 and cost > 0:
        daily_avg = cost / active_days
        month_projection = daily_avg * 30
        lines += [
            f"",
            f"## 成本预测",
            f"",
            f"| 指标 | 数值 |",
            f"|------|------|",
            f"| 日均费用 | {_fmt_cost(daily_avg)} |",
            f"| 月度预测 | {_fmt_cost(month_projection)} |",
            f"| 活跃天数 | {active_days}/{days} 天 |",
        ]

    # 效率指标
    total_tokens = merged.token_usage.total
    total_code = merged.total_added + merged.total_removed
    avg_tokens_per_msg = total_tokens // max(merged.user_message_count, 1)
    code_per_1k_token = round(total_code / max(total_tokens / 1000, 1), 2)
    active_secs = merged.active_duration.total_seconds()
    ai_secs = merged.ai_duration.total_seconds()
    ai_ratio = round(ai_secs / max(active_secs, 1) * 100)

    # 效率评分 (0-100)
    # 代码产出 (40分): code_per_1k_token, 0.5以上满分
    code_score = min(40, int(code_per_1k_token / 0.5 * 40))
    # 指令精准 (30分): avg_tokens_per_msg 越低越好, 50K以下满分
    precision_score = max(0, min(30, int((1 - min(avg_tokens_per_msg, 200_000) / 200_000) * 30)))
    # AI利用率 (30分): 70%以上满分
    util_score = min(30, int(ai_ratio / 70 * 30))
    total_score = code_score + precision_score + util_score

    grade = "S" if total_score >= 90 else "A" if total_score >= 75 else "B" if total_score >= 60 else "C" if total_score >= 40 else "D"

    lines += [
        f"",
        f"## 效率评分",
        f"",
        f"**{grade} ({total_score}/100)**",
        f"",
        f"| 维度 | 数值 | 得分 |",
        f"|------|------|------|",
        f"| 代码产出率 | {code_per_1k_token} 行/K Token | {code_score}/40 |",
        f"| 指令精准度 | {_fmt_tokens(avg_tokens_per_msg)} Token/条 | {precision_score}/30 |",
        f"| AI 利用率 | {ai_ratio}% | {util_score}/30 |",
    ]

    lines += [
        f"",
        f"---",
        f"*Generated by [cc-statistics](https://github.com/androidZzT/cc-statistics)*",
    ]

    return "\n".join(lines)



"""分析会话数据，计算各项工程指标"""

from __future__ import annotations

import os
import subprocess
from collections import Counter, defaultdict
from dataclasses import dataclass, field
from datetime import datetime, timedelta, timezone
from pathlib import Path

from .parser import Message, Session

# 文件扩展名 → 语言映射
EXT_TO_LANG: dict[str, str] = {
    ".py": "Python",
    ".js": "JavaScript",
    ".ts": "TypeScript",
    ".tsx": "TypeScript (JSX)",
    ".jsx": "JavaScript (JSX)",
    ".java": "Java",
    ".kt": "Kotlin",
    ".kts": "Kotlin Script",
    ".swift": "Swift",
    ".go": "Go",
    ".rs": "Rust",
    ".c": "C",
    ".cpp": "C++",
    ".h": "C/C++ Header",
    ".cs": "C#",
    ".rb": "Ruby",
    ".php": "PHP",
    ".scala": "Scala",
    ".sh": "Shell",
    ".bash": "Shell",
    ".zsh": "Shell",
    ".html": "HTML",
    ".css": "CSS",
    ".scss": "SCSS",
    ".less": "Less",
    ".json": "JSON",
    ".yaml": "YAML",
    ".yml": "YAML",
    ".toml": "TOML",
    ".xml": "XML",
    ".sql": "SQL",
    ".md": "Markdown",
    ".r": "R",
    ".lua": "Lua",
    ".dart": "Dart",
    ".vue": "Vue",
    ".svelte": "Svelte",
    ".gradle": "Gradle",
}

# 工具说明
TOOL_DESCRIPTIONS: dict[str, str] = {
    "Bash": "执行 Shell 命令",
    "Read": "读取文件内容",
    "Write": "创建/覆写文件",
    "Edit": "编辑文件（精确替换）",
    "Glob": "按模式搜索文件",
    "Grep": "按内容搜索文件",
    "Agent": "启动子代理执行子任务",
    "Skill": "调用技能/Slash命令",
    "WebFetch": "抓取网页内容",
    "WebSearch": "搜索互联网",
    "NotebookEdit": "编辑 Jupyter Notebook",
    "LSP": "调用语言服务器",
    "TodoWrite": "写入待办事项",
    "AskUserQuestion": "向用户提问",
    "TaskCreate": "创建任务",
    "TaskUpdate": "更新任务状态",
    "TaskGet": "获取任务信息",
    "TaskList": "列出任务",
    "TaskOutput": "获取任务输出",
    "TaskStop": "停止任务",
    "ToolSearch": "搜索可用工具",
    "SendMessage": "向子代理发送消息",
}

# 活跃时间判定：两条消息间隔超过此值视为"不活跃"
IDLE_THRESHOLD = timedelta(minutes=5)


def _parse_ts(ts: str) -> datetime | None:
    """解析 ISO 格式或毫秒时间戳"""
    if not ts:
        return None
    try:
        if isinstance(ts, (int, float)) or ts.isdigit():
            return datetime.fromtimestamp(int(ts) / 1000, tz=timezone.utc)
        # ISO format
        ts = ts.replace("Z", "+00:00")
        return datetime.fromisoformat(ts)
    except (ValueError, OSError):
        return None


def _detect_lang(file_path: str) -> str:
    """根据文件扩展名检测编程语言"""
    _, ext = os.path.splitext(file_path)
    return EXT_TO_LANG.get(ext.lower(), f"Other ({ext})" if ext else "Unknown")


def _count_lines(text: str) -> int:
    """统计文本行数（不含末尾空行）"""
    if not text:
        return 0
    return len(text.rstrip("\n").split("\n"))


@dataclass
class CodeChange:
    file_path: str
    language: str
    added: int = 0
    removed: int = 0


@dataclass
class TokenUsage:
    input_tokens: int = 0
    output_tokens: int = 0
    cache_read_input_tokens: int = 0
    cache_creation_input_tokens: int = 0

    @property
    def total(self) -> int:
        return (
            self.input_tokens
            + self.output_tokens
            + self.cache_read_input_tokens
            + self.cache_creation_input_tokens
        )


@dataclass
class SessionStats:
    """单个会话的统计结果"""
    session_id: str
    project_path: str

    # 1. 用户指令数
    user_message_count: int = 0

    # 2. 工具调用
    tool_call_total: int = 0
    tool_call_counts: dict[str, int] = field(default_factory=dict)

    # 3. 开发时长
    start_time: datetime | None = None
    end_time: datetime | None = None
    total_duration: timedelta = field(default_factory=timedelta)
    ai_duration: timedelta = field(default_factory=timedelta)       # AI 处理时长
    user_duration: timedelta = field(default_factory=timedelta)     # 用户活跃时长（审查/编码）
    active_duration: timedelta = field(default_factory=timedelta)   # ai + user
    turn_count: int = 0                                             # 对话轮次数

    # 4. 代码行数 (AI — 来自 Edit/Write 工具调用)
    code_changes: list[CodeChange] = field(default_factory=list)
    lines_by_lang: dict[str, dict[str, int]] = field(default_factory=dict)
    total_added: int = 0
    total_removed: int = 0

    # 4b. 代码行数 (Git — 会话期间的所有 commit)
    git_total_added: int = 0
    git_total_removed: int = 0
    git_lines_by_lang: dict[str, dict[str, int]] = field(default_factory=dict)
    git_commit_count: int = 0
    git_ai_commit_count: int = 0       # Co-Authored-By: Claude 的 commit 数
    git_ai_added: int = 0              # AI commit 的新增行数
    git_ai_removed: int = 0            # AI commit 的删除行数
    git_available: bool = False

    # 5. Token 消耗
    token_usage: TokenUsage = field(default_factory=TokenUsage)
    token_by_model: dict[str, TokenUsage] = field(default_factory=dict)


@dataclass
class GitStats:
    total_added: int = 0
    total_removed: int = 0
    commit_count: int = 0
    ai_commit_count: int = 0
    ai_added: int = 0
    ai_removed: int = 0
    lines_by_lang: dict[str, dict[str, int]] = field(default_factory=dict)


# AI commit 检测关键词（commit message 中包含这些表示 AI 参与）
_AI_COMMIT_MARKERS = [
    "co-authored-by: claude",
    "co-authored-by: cursor",
    "co-authored-by: github copilot",
    "co-authored-by: codex",
    "co-authored-by: gemini",
    "generated by ai",
    "generated with claude",
    "generated by claude",
]


def _collect_git_stats(
    project_path: str,
    start_time: datetime,
    end_time: datetime,
) -> GitStats:
    """通过 git log 收集会话时间段内的 commit 变更统计，区分 AI/人工 commit"""
    repo_dir = Path(project_path)
    if not (repo_dir / ".git").exists() and not (repo_dir / ".git").is_file():
        return GitStats()

    # 转为本地时间，前后各扩展 1 分钟避免边界问题
    local_start = (start_time - timedelta(minutes=1)).astimezone()
    local_end = (end_time + timedelta(minutes=1)).astimezone()
    since = local_start.strftime("%Y-%m-%dT%H:%M:%S")
    until = local_end.strftime("%Y-%m-%dT%H:%M:%S")

    # 用 --format 分隔 hash 和 commit body（用 %x00 作为分隔符）
    try:
        result = subprocess.run(
            [
                "git", "log",
                "--numstat",
                "--format=%x00%H%n%B%x00",
                f"--since={since}",
                f"--until={until}",
            ],
            cwd=project_path,
            capture_output=True,
            text=True,
            timeout=10,
        )
        if result.returncode != 0:
            return GitStats()
    except (subprocess.TimeoutExpired, FileNotFoundError):
        return GitStats()

    stats = GitStats()
    lang_stats: dict[str, dict[str, int]] = defaultdict(
        lambda: {"added": 0, "removed": 0}
    )

    # 按 commit 分段解析
    is_ai_commit = False
    commit_added = 0
    commit_removed = 0

    for line in result.stdout.splitlines():
        line_stripped = line.strip()
        if not line_stripped:
            continue

        # 检测 commit 分隔符（\x00HASH\n...body...\x00）
        if "\x00" in line:
            # 先结算上一个 commit 的统计
            if stats.commit_count > 0 and is_ai_commit:
                stats.ai_added += commit_added
                stats.ai_removed += commit_removed

            # 提取 commit body，判断是否 AI commit
            clean = line.replace("\x00", "")
            if len(clean) >= 40:
                stats.commit_count += 1
                commit_added = 0
                commit_removed = 0
                is_ai_commit = False
            # 检查 body 中的 AI 标记
            lower = line.lower()
            if any(marker in lower for marker in _AI_COMMIT_MARKERS):
                is_ai_commit = True
                stats.ai_commit_count += 1
            continue

        # 检查非分隔行中的 AI 标记（commit body 可能跨多行）
        lower = line_stripped.lower()
        if any(marker in lower for marker in _AI_COMMIT_MARKERS):
            if not is_ai_commit:
                is_ai_commit = True
                stats.ai_commit_count += 1

        # numstat 行：added\tremoved\tfile_path
        parts = line_stripped.split("\t")
        if len(parts) == 3:
            added_str, removed_str, file_path = parts
            if added_str == "-" or removed_str == "-":
                continue
            try:
                added = int(added_str)
                removed = int(removed_str)
            except ValueError:
                continue
            stats.total_added += added
            stats.total_removed += removed
            commit_added += added
            commit_removed += removed
            lang = _detect_lang(file_path)
            lang_stats[lang]["added"] += added
            lang_stats[lang]["removed"] += removed

    # 结算最后一个 commit
    if is_ai_commit:
        stats.ai_added += commit_added
        stats.ai_removed += commit_removed

    stats.lines_by_lang = dict(lang_stats)
    return stats


def analyze_session(session: Session) -> SessionStats:
    """分析单个会话，返回统计结果"""
    stats = SessionStats(
        session_id=session.session_id,
        project_path=session.project_path,
    )

    # 构建带时间戳的消息序列，用于时长分析
    # timed_msgs: list of (datetime, role)
    # role: "user_real" = 真实用户消息, "user_tool" = 工具返回, "assistant"
    timed_msgs: list[tuple[datetime, str]] = []

    for msg in session.messages:
        ts = _parse_ts(msg.timestamp)
        if not ts:
            continue

        if msg.role == "user":
            if msg.is_tool_result or msg.is_meta:
                timed_msgs.append((ts, "user_tool"))
            else:
                timed_msgs.append((ts, "user_real"))
        elif msg.role == "assistant":
            timed_msgs.append((ts, "assistant"))

        # -------- 1. 用户指令数 --------
        if msg.role == "user" and not msg.is_tool_result and not msg.is_meta:
            stats.user_message_count += 1

        # -------- 2. 工具调用 --------
        if msg.role == "assistant":
            for tc in msg.tool_calls:
                stats.tool_call_total += 1

                # 展开 Skill 和 MCP 工具为具体名称
                display_name = tc.name
                if tc.name == "Skill":
                    skill_name = tc.input.get("skill", "")
                    if skill_name:
                        display_name = f"Skill:{skill_name}"
                elif tc.name.startswith("mcp__"):
                    # mcp__server__method → MCP:server/method
                    parts = tc.name.split("__")
                    if len(parts) >= 3:
                        display_name = f"MCP:{parts[1]}/{parts[2]}"

                stats.tool_call_counts[display_name] = (
                    stats.tool_call_counts.get(display_name, 0) + 1
                )

                # -------- 4. 代码行数（从 Edit/Write 工具提取） --------
                if tc.name == "Write":
                    fp = tc.input.get("file_path", "")
                    content = tc.input.get("content", "")
                    lang = _detect_lang(fp)
                    added = _count_lines(content)
                    change = CodeChange(
                        file_path=fp, language=lang, added=added, removed=0
                    )
                    stats.code_changes.append(change)

                elif tc.name == "Edit":
                    fp = tc.input.get("file_path", "")
                    old = tc.input.get("old_string", "")
                    new = tc.input.get("new_string", "")
                    lang = _detect_lang(fp)
                    old_lines = _count_lines(old)
                    new_lines = _count_lines(new)
                    change = CodeChange(
                        file_path=fp,
                        language=lang,
                        added=new_lines,
                        removed=old_lines,
                    )
                    stats.code_changes.append(change)

            # -------- 5. Token 消耗 --------
            usage = msg.usage
            if usage:
                inp = usage.get("input_tokens", 0)
                out = usage.get("output_tokens", 0)
                cache_read = usage.get("cache_read_input_tokens", 0)
                cache_create = usage.get("cache_creation_input_tokens", 0)

                stats.token_usage.input_tokens += inp
                stats.token_usage.output_tokens += out
                stats.token_usage.cache_read_input_tokens += cache_read
                stats.token_usage.cache_creation_input_tokens += cache_create

                model = msg.model or ""
                if not model or model.startswith("<"):
                    model = "unknown"
                if model not in stats.token_by_model:
                    stats.token_by_model[model] = TokenUsage()
                m = stats.token_by_model[model]
                m.input_tokens += inp
                m.output_tokens += out
                m.cache_read_input_tokens += cache_read
                m.cache_creation_input_tokens += cache_create

    # -------- 3. 时长计算（基于对话轮次） --------
    # 一轮 = 用户发消息 → AI 处理（可能多次工具调用）→ AI 最终回复
    # AI 时长 = 每轮中从用户消息到 AI 最后一条响应
    # 用户时长 = 上一轮 AI 最后响应到本轮用户消息（超过阈值视为离开）
    if timed_msgs:
        stats.start_time = timed_msgs[0][0]
        stats.end_time = timed_msgs[-1][0]

        ai_total = timedelta()
        user_total = timedelta()
        turn_count = 0

        # 将消息流切分为轮次：每遇到一条 user_real 开启新轮
        # turn_start: 本轮用户消息的时间
        # last_ai_end: 上一轮 AI 最后响应的时间
        turn_start: datetime | None = None
        turn_last_ai: datetime | None = None
        last_ai_end: datetime | None = None  # 上一轮结束

        for ts, role in timed_msgs:
            if role == "user_real":
                # 结算上一轮的 AI 时长
                if turn_start is not None and turn_last_ai is not None:
                    ai_total += turn_last_ai - turn_start
                    turn_count += 1

                # 计算用户时长（上一轮 AI 结束 → 本轮用户消息）
                if last_ai_end is not None:
                    gap = ts - last_ai_end
                    if gap <= IDLE_THRESHOLD:
                        user_total += gap

                # 上一轮终点
                if turn_last_ai is not None:
                    last_ai_end = turn_last_ai

                turn_start = ts
                turn_last_ai = None
            elif role in ("assistant", "user_tool"):
                # AI 响应或工具返回，都算 AI 工作中
                turn_last_ai = ts

        # 结算最后一轮
        if turn_start is not None and turn_last_ai is not None:
            ai_total += turn_last_ai - turn_start
            turn_count += 1

        stats.ai_duration = ai_total
        stats.user_duration = user_total
        stats.active_duration = ai_total + user_total
        # total_duration = 活跃时长（而非首尾差），避免 resume 会话跨天导致虚高
        stats.total_duration = ai_total + user_total
        stats.turn_count = turn_count

    # -------- 4. 按语言汇总 --------
    lang_stats: dict[str, dict[str, int]] = defaultdict(
        lambda: {"added": 0, "removed": 0}
    )
    for change in stats.code_changes:
        lang_stats[change.language]["added"] += change.added
        lang_stats[change.language]["removed"] += change.removed
        stats.total_added += change.added
        stats.total_removed += change.removed
    stats.lines_by_lang = dict(lang_stats)

    # -------- 4b. Git 变更统计 --------
    if stats.start_time and stats.end_time and session.project_path:
        git = _collect_git_stats(
            session.project_path, stats.start_time, stats.end_time
        )
        if git.commit_count > 0:
            stats.git_available = True
            stats.git_total_added = git.total_added
            stats.git_total_removed = git.total_removed
            stats.git_commit_count = git.commit_count
            stats.git_ai_commit_count = git.ai_commit_count
            stats.git_ai_added = git.ai_added
            stats.git_ai_removed = git.ai_removed
            stats.git_lines_by_lang = git.lines_by_lang

    return stats


def merge_stats(all_stats: list[SessionStats]) -> SessionStats:
    """合并多个会话的统计结果"""
    merged = SessionStats(session_id="merged", project_path="all")

    all_starts = []
    all_ends = []

    for s in all_stats:
        merged.user_message_count += s.user_message_count
        merged.tool_call_total += s.tool_call_total

        for name, count in s.tool_call_counts.items():
            merged.tool_call_counts[name] = merged.tool_call_counts.get(name, 0) + count

        merged.ai_duration += s.ai_duration
        merged.user_duration += s.user_duration
        merged.active_duration += s.active_duration
        merged.turn_count += s.turn_count

        if s.start_time:
            all_starts.append(s.start_time)
        if s.end_time:
            all_ends.append(s.end_time)

        merged.code_changes.extend(s.code_changes)
        merged.total_added += s.total_added
        merged.total_removed += s.total_removed

        for lang, counts in s.lines_by_lang.items():
            if lang not in merged.lines_by_lang:
                merged.lines_by_lang[lang] = {"added": 0, "removed": 0}
            merged.lines_by_lang[lang]["added"] += counts["added"]
            merged.lines_by_lang[lang]["removed"] += counts["removed"]

        # Git 变更
        if s.git_available:
            merged.git_available = True
            merged.git_total_added += s.git_total_added
            merged.git_total_removed += s.git_total_removed
            merged.git_commit_count += s.git_commit_count
            merged.git_ai_commit_count += s.git_ai_commit_count
            merged.git_ai_added += s.git_ai_added
            merged.git_ai_removed += s.git_ai_removed
            for lang, counts in s.git_lines_by_lang.items():
                if lang not in merged.git_lines_by_lang:
                    merged.git_lines_by_lang[lang] = {"added": 0, "removed": 0}
                merged.git_lines_by_lang[lang]["added"] += counts["added"]
                merged.git_lines_by_lang[lang]["removed"] += counts["removed"]

        merged.token_usage.input_tokens += s.token_usage.input_tokens
        merged.token_usage.output_tokens += s.token_usage.output_tokens
        merged.token_usage.cache_read_input_tokens += s.token_usage.cache_read_input_tokens
        merged.token_usage.cache_creation_input_tokens += s.token_usage.cache_creation_input_tokens

        for model, usage in s.token_by_model.items():
            if model not in merged.token_by_model:
                merged.token_by_model[model] = TokenUsage()
            m = merged.token_by_model[model]
            m.input_tokens += usage.input_tokens
            m.output_tokens += usage.output_tokens
            m.cache_read_input_tokens += usage.cache_read_input_tokens
            m.cache_creation_input_tokens += usage.cache_creation_input_tokens

    if all_starts:
        merged.start_time = min(all_starts)
    if all_ends:
        merged.end_time = max(all_ends)
    # total_duration = 活跃时长之和，而非首尾差
    merged.total_duration = merged.active_duration

    return merged
